import Matter from 'matter-js'

const {
  Render,
  Engine,
  World,
  Runner,
  Body,
  Bodies,
  Constraint,
  Composite,
  Composites,
  Common,
  Mouse,
  MouseConstraint,
  Events,
} = Matter;

export default class {
  constructor(selector, imageList, pixiView, isDevice) {
    this.el = document.querySelector(selector);
    this.pixiView = pixiView;

    const resolution = this.getResolution;
    const engine = Engine.create();
    const world = engine.world;

    const render = Render.create({
      element: this.el,
      engine: engine,
      options: {
        width: resolution.x,
        height: resolution.y,
        pixelRatio: window.devicePixelRatio || 1,
        hasBounds: false,
        enabled: true,
        showSleeping: false,
        showDebug: false,
        showBroadphase: false,
        showBounds: false,
        showVelocity: false,
        showCollisions: false,
        showSeparations: false,
        showAxes: false,
        showPosition: false,
        showAngleIndicator: false,
        showIds: false,
        showShadows: false,
        showVertexNumbers: false,
        showConvexHulls: false,
        showInternalEdges: false,
        showMousePosition: false,
        background: 'transparent',
        wireframeBackground: 'transparent',
        wireframes: false,
      }
    });
    Render.lookAt(render, {
      min: { x: resolution.x * 0, y: resolution.y * 0 },
      max: { x: resolution.x, y: resolution.y }
    });
    Render.run(render);

    const runner = Runner.create({});
    Runner.run(runner, engine);


    // 中心の縁を描画
    const circleRadius = isDevice
      ? (window.innerWidth - 75) / 2
      : (window.innerHeight - 240) / 2;
    const kvCircle = Bodies.circle(
      window.innerWidth / 2,
      window.innerHeight / 2,
      circleRadius,
      {
        isStatic: true,
        render: {
          visible: false,
        }
      }
    );
    World.add(world, kvCircle);

    // アンカースタイルを設定
    const _anchor = (x, y, bodyB) => {
      return Constraint.create({
        pointA: {x: x, y: y},
        bodyB: bodyB,
        stiffness: 1,
        render: { visible: false }
      });
    }
    const particleOptions = {
      friction: 0.05,
      frictionStatic: 0.1,
      render: { visible: false }
    }
    this.imageList = [...imageList].map(el => {
      const property = this.getProperty(el);
      const magnification = {
        x: property.w / (property.w - property.r * 2),
        y: property.h / (property.h - property.r * 2),
      };
      const pointA = { x: property.x, y: property.y };
      const pointB = { x: property.x + property.w, y: property.y };
      const composites =  Composites.softBody(
        property.x,
        property.y,
        4,
        3,
        0,
        0,
        true,
        property.r,
        particleOptions
      );
      composites.constraints.forEach(x => {
        Body.set(x, { render: { visible: false } });
      });
      const anchorA =  _anchor(pointA.x, pointA.y, composites.bodies[0]);
      const anchorB =  _anchor(pointB.x, pointB.y, composites.bodies[3]);
      World.add(world, [anchorA, anchorB]);
      return { composites: composites, property: property,  magnification: magnification};
    });
    World.add(world, this.imageList.map(x => x.composites));

    Events.on(engine, 'afterUpdate', () => {
      this.imageList.forEach((x, i) => {
        const target = this.pixiView.imageList[i];
        this._render(x, target);
      })
    });


    const _create = () => {
      const x = resolution.x * Math.random();
      const circle = Bodies.circle(x, -100, Common.random(10, 25), {
        restitution: 1,
        friction: 0.1,
        render: {
          fillStyle: '#2a2a2a'
        }
      });
      World.add(world, circle);
    }
    // 6秒ごとにボールを落とす
    setInterval(() => _create(), 6000);
  }

  _render(o, mesh) {
    if(mesh.vertexData.length < 1) return;
    const bodies = o.composites.bodies;
    const magnification = o.magnification;
    const property = o.property;

    bodies.forEach((vertex, i) => {
      i *= 2;

      const point = vertex.position;
      const x = point.x - (property.x + property.r);
      const y = point.y - (property.y + property.r);

      // PIXI.SimplePlaneの頂点座標を更新
      mesh.vertexData[i] = x * magnification.x + property.x;
      mesh.vertexData[i + 1] = y * magnification.y + property.y;
    });
  }

  getProperty(el) {
    const w = el.clientWidth;
    const h = el.clientHeight;
    const x = el.x;
    const y = el.y;
    const r = w / 8;
    return { x, y, w, h, r }
  }

  get getResolution() {
    return {
      x: this.el.clientWidth,
      y: this.el.clientHeight,
    }
  }
}
